﻿using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using SimpleJSON;
using MVR.FileManagementSecure;
using UnityEngine.UI;

namespace JayJayWon
{
    public class Settings
    {

        public static JSONStorableFloat maxButtonsPerPageVRJSF = new JSONStorableFloat("maxButtonsPerPageVR", 200f,MaxButtonsPerPageVRCallback, 20f,2000f);
        public static JSONStorableFloat maxButtonsPerPageDesktopJSF = new JSONStorableFloat("maxButtonsPerPageDesktop", 200f, MaxButtonsPerPageDesktopCallback, 20f, 2000f);

        public static float maxButtonsPerPage { get
            {
                if (Globals.isVREnabled) return maxButtonsPerPageVRJSF.val;
                else return maxButtonsPerPageDesktopJSF.val;
            } }

        public static JSONStorableBool selectAtomOnLoadJSB = new JSONStorableBool("selectAtomOnLoad", true,SettingsBoolChange);

        public static JSONStorableBool keepFolderBrowserOpenJSB = new JSONStorableBool("keepFolderBrowserOpen", false, SettingsBoolChange);

        public static JSONStorableFloat uiScaleJSF = new JSONStorableFloat("Desktop UI Scale", 1f, UIScaleCallback, 0.5f, 2f);
        public static JSONStorableEnumStringChooser resourceNameSearchModeJSESC = new JSONStorableEnumStringChooser("resourceNameSearchMode", ResourceNameSearchMode.enumManifestName,ResourceNameSearchMode.ContainsFullPath,"Resource Filter Name Search Mode", ResourceNameSearchModeCallback);
        public static JSONStorableEnumStringChooser launchButtonPlacementJSESC = new JSONStorableEnumStringChooser("launchButtonPlacement", VAMUILaunchButtonPosition.enumManifestName, VAMUILaunchButtonPosition.sceneBrowserSub, "BrowserAssist Launch Button Position on VAM UI", LaunchButtonPlacementCallback);

        public static JSONStorableEnumStringChooser defaultTargetPersonTypeJSE = new JSONStorableEnumStringChooser("defaultTargetPersonType",PersonGenderTypes.enumManifestName,PersonGenderTypes.firstFemale,"Default Target Person Type");

        private static int _resourceButtonSizeVR=4;
        private static int _resourceButtonSizeDesktop=4;
        public static int resourceButtonSizeVR { get{ return _resourceButtonSizeVR; } set { _resourceButtonSizeVR = value;SaveSettings(); } }
        public static int resourceButtonSizeDesktop { get { return _resourceButtonSizeDesktop; } set { _resourceButtonSizeDesktop = value; SaveSettings(); } }

        public static JSONStorableBool enableTopScreenLaunchButtonJSB = new JSONStorableBool("EnableTopScreenLaunchButton", true, EnableTopScreenLaunchButtonChange);
        public static JSONStorableBool enableBulkDeleteJSB = new JSONStorableBool("EnableBulkDelete", false, SettingsBoolChange);

        public static JSONStorableEnumStringChooser defaultResourceTypeJESC = new JSONStorableEnumStringChooser("defaultResourceType", ResourceType.enumManifestName, ResourceType.scene, "Initial Resource Type on BrowserAssist startup",SettingsStringChange);
        public static JSONStorableEnumStringChooser creatorFilterOrderJSEC = new JSONStorableEnumStringChooser("creatorFilterOrder",CreatorFilterSortOrder.enumManifestName,CreatorFilterSortOrder.resourceCount,"Order for Creator Filter choices", SettingsStringChange);

        public static RelativeAtomPositionRotation desktopDefaultNewAtomPosRot = new RelativeAtomPositionRotation();
        public static RelativeAtomPositionRotation vrDefaultNewAtomPosRot = new RelativeAtomPositionRotation();

        public static JSONStorableBool dockToVAMUIJSB = new JSONStorableBool("DockToVAMUI", true, SettingsBoolChange);

        public static JSONStorableBool disableMetaPurgeOnMissingVARJSB = new JSONStorableBool("disableMetaPurgeOnMissingVAR", false, SettingsBoolChange);

        public static Dictionary<int, ResourceSettings> resourceSettings = new Dictionary<int, ResourceSettings>();

        private static bool restoreInProgress = false;

        public Settings()
        {
            
        }

        private static void EnableTopScreenLaunchButtonChange(bool i)
        {
            SaveSettings();
            if (!restoreInProgress) Globals.mvrScript.ToggleTopMenuLanchButton();
        }

        
        private static void LaunchButtonPlacementCallback(string val)
        {
            SaveSettings();
            if (!restoreInProgress) Globals.mvrScript.RePositionLaunchButton();
        }
        private static void MaxButtonsPerPageVRCallback(float val)
        {
            maxButtonsPerPageVRJSF.valNoCallback = Mathf.Round(val / 10f) *10f;
            if (Globals.isVREnabled &&
                BrowserAssist.mainBrowserUI!=null && BrowserAssist.mainBrowserUI.resourceBrowserUI != null && BrowserAssist.mainBrowserUI.isVisible) BrowserAssist.mainBrowserUI.RefreshButtonLayout();
            if (Globals.isVREnabled &&
                BrowserAssist.callbackBrowserUI != null && BrowserAssist.callbackBrowserUI.resourceBrowserUI != null && BrowserAssist.callbackBrowserUI.isVisible) BrowserAssist.callbackBrowserUI.RefreshButtonLayout();
            SaveSettings();
        }

        private static void MaxButtonsPerPageDesktopCallback(float val)
        {
            maxButtonsPerPageDesktopJSF.valNoCallback = Mathf.Round(val / 10f) * 10f;
            if (!Globals.isVREnabled &&
                BrowserAssist.mainBrowserUI != null && BrowserAssist.mainBrowserUI.resourceBrowserUI != null && BrowserAssist.mainBrowserUI.isVisible) BrowserAssist.mainBrowserUI.RefreshButtonLayout();
            if (!Globals.isVREnabled &&
                BrowserAssist.callbackBrowserUI != null && BrowserAssist.callbackBrowserUI.resourceBrowserUI != null && BrowserAssist.callbackBrowserUI.isVisible) BrowserAssist.callbackBrowserUI.RefreshButtonLayout();
            SaveSettings();
        }

        private static void ResourceNameSearchModeCallback(string val)
        {
            if (BrowserAssist.mainBrowserUI != null) BrowserAssist.mainBrowserUI.browserFiltersUI.SetResourceNameFilterHelpText();
            if (BrowserAssist.callbackBrowserUI != null) BrowserAssist.callbackBrowserUI.browserFiltersUI.SetResourceNameFilterHelpText();
            SaveSettings();
        }
        private static void UIScaleCallback(float val)
        {
            if (BrowserAssist.mainBrowserUI != null) BrowserAssist.mainBrowserUI.RefreshDesktopScale();
            if (BrowserAssist.callbackBrowserUI != null) BrowserAssist.callbackBrowserUI.RefreshDesktopScale();
            SaveSettings();
        }
        private static void SettingsBoolChange(bool val)
        {
            SaveSettings();
        }

        private static void SettingsFloatChange(float val)
        {
            SaveSettings();
        }

        private static void SettingsStringChange(string val)
        {
            SaveSettings();
        }

        private static void SettingsIntChange(int val)
        {
            SaveSettings();
        }

        public static void SaveSettings()
        {
            if (!restoreInProgress)
            {
                JSONClass settingsJC = new JSONClass();

                maxButtonsPerPageVRJSF.StoreJSON(settingsJC["VRSettings"].AsObject);
                maxButtonsPerPageDesktopJSF.StoreJSON(settingsJC["DesktopSettings"].AsObject);
                uiScaleJSF.StoreJSON(settingsJC["DesktopSettings"].AsObject);
                resourceNameSearchModeJSESC.StoreJSON(settingsJC["FilterSettings"].AsObject);
                creatorFilterOrderJSEC.StoreJSON(settingsJC["FilterSettings"].AsObject);
                launchButtonPlacementJSESC.StoreJSON(settingsJC["GeneralSettings"].AsObject);
                settingsJC["BrowserSettings"]["ResourceButtonSizeVR"].AsInt = _resourceButtonSizeVR;
                settingsJC["BrowserSettings"]["ResourceButtonSizeDesktop"].AsInt = _resourceButtonSizeDesktop;
                defaultResourceTypeJESC.StoreJSON(settingsJC["BrowserSettings"].AsObject);
                enableTopScreenLaunchButtonJSB.StoreJSON(settingsJC["GeneralSettings"].AsObject);
                enableBulkDeleteJSB.StoreJSON(settingsJC["GeneralSettings"].AsObject);
                

                selectAtomOnLoadJSB.StoreJSON(settingsJC["ResourceActionSettings"].AsObject);
                keepFolderBrowserOpenJSB.StoreJSON(settingsJC["ResourceActionSettings"].AsObject);

                desktopDefaultNewAtomPosRot.StoreJSON(settingsJC["DesktopSettings"]["DefaultNewAtomPosRot"].AsObject);
                vrDefaultNewAtomPosRot.StoreJSON(settingsJC["VRSettings"]["DefaultNewAtomPosRot"].AsObject);

                dockToVAMUIJSB.StoreJSON(settingsJC["VRSettings"].AsObject);
                disableMetaPurgeOnMissingVARJSB.StoreJSON(settingsJC["GeneralSettings"].AsObject);

                foreach (int resourceType in EnumManifest.GetEnumsList(ResourceType.enumManifestName))
                {
                    string resourceTypeStoreLabel = EnumManifest.GetStoreVal(ResourceType.enumManifestName, resourceType);

                    settingsJC["ResourceSettings"][resourceTypeStoreLabel] = resourceSettings[resourceType].GetJSON();
                }

                SuperController.singleton.SaveJSON(settingsJC, Const.BAScriptDataFolder + "\\" + "BASettings.cfg");
            }

        }

        public static JSONClass GetSettingsJSON()
        {
            JSONClass settingsJSON = null;

            if (FileManagerSecure.FileExists(Const.BAScriptDataFolder + "\\" + "BASettings.cfg"))
            {

                JSONClass settingsJC;
                try
                {
                    settingsJC = SuperController.singleton.LoadJSON(Const.BAScriptDataFolder + "\\" + "BASettings.cfg").AsObject;
                    return settingsJC;
                }
                catch
                {
                    Utils.LogDebug("Settings.LoadSettings: Error during load of settings config file. Possible malformed JSON");
                    return null;
                }
            }

            return settingsJSON;
        }

        public static void EarlyLoadSettings(JSONClass settingsJC)
        {
            if (settingsJC != null)
            {
                restoreInProgress = true;
                disableMetaPurgeOnMissingVARJSB.RestoreFromJSON(settingsJC["GeneralSettings"].AsObject);
                restoreInProgress = false;
            }
                
        }

        public static void LoadSettings(JSONClass settingsJC)
        {
            List<int> resourceTypesToExclude = new List<int>();
            if (!PC.patreonContentEnabled)
            {
                resourceTypesToExclude = EnumManifest.GetEnumsList(ResourceType.enumManifestName);
                resourceTypesToExclude.Remove(ResourceType.scene);
                resourceTypesToExclude.Remove(ResourceType.presetAppearance);
            }
            desktopDefaultNewAtomPosRot.Init(SettingsIntChange, SettingsIntChange, SettingsStringChange);
            desktopDefaultNewAtomPosRot.InitSettingsCallback(SettingsFloatChange, SettingsIntChange);
            vrDefaultNewAtomPosRot.Init(SettingsIntChange, SettingsIntChange, SettingsStringChange);
            vrDefaultNewAtomPosRot.InitSettingsCallback(SettingsFloatChange, SettingsIntChange);

            defaultResourceTypeJESC.SetEnumChoices(ResourceType.enumManifestName, resourceTypesToExclude);

            if (settingsJC!=null)
            {
                restoreInProgress = true;

                foreach (int resourceType in EnumManifest.GetEnumsList(ResourceType.enumManifestName))
                {
                    ResourceSettings newResourceSettings = new ResourceSettings(resourceType);
                    
                    string resourceTypeStoreLabel = EnumManifest.GetStoreVal(ResourceType.enumManifestName, resourceType);

                    newResourceSettings.RestoreJSON(settingsJC["ResourceSettings"][resourceTypeStoreLabel].AsObject);

                    resourceSettings.Add(resourceType, newResourceSettings);
                }

                uiScaleJSF.RestoreFromJSON(settingsJC["DesktopSettings"].AsObject);
                maxButtonsPerPageVRJSF.RestoreFromJSON(settingsJC["VRSettings"].AsObject);
                maxButtonsPerPageDesktopJSF.RestoreFromJSON(settingsJC["DesktopSettings"].AsObject);

                if (settingsJC["BrowserSettings"]["Maximum Number of Resources per Browser page"] != null)
                {
                    maxButtonsPerPageVRJSF.valNoCallback = settingsJC["BrowserSettings"]["Maximum Number of Resources per Browser page"].AsFloat;
                    maxButtonsPerPageDesktopJSF.valNoCallback = settingsJC["BrowserSettings"]["Maximum Number of Resources per Browser page"].AsFloat;
                }

                resourceNameSearchModeJSESC.RestoreFromJSON(settingsJC["FilterSettings"].AsObject);
                creatorFilterOrderJSEC.RestoreFromJSON(settingsJC["FilterSettings"].AsObject);

                launchButtonPlacementJSESC.RestoreFromJSON(settingsJC["GeneralSettings"].AsObject);
                if (settingsJC["BrowserSettings"]["ResourceButtonSizeVR"]!=null) _resourceButtonSizeVR = settingsJC["BrowserSettings"]["ResourceButtonSizeVR"].AsInt;
                if (settingsJC["BrowserSettings"]["ResourceButtonSizeDesktop"] != null) _resourceButtonSizeDesktop = settingsJC["BrowserSettings"]["ResourceButtonSizeDesktop"].AsInt;
                selectAtomOnLoadJSB.RestoreFromJSON(settingsJC["ResourceActionSettings"].AsObject);
                keepFolderBrowserOpenJSB.RestoreFromJSON(settingsJC["ResourceActionSettings"].AsObject);

                defaultResourceTypeJESC.RestoreFromJSON(settingsJC["BrowserSettings"].AsObject);
                enableTopScreenLaunchButtonJSB.RestoreFromJSON(settingsJC["GeneralSettings"].AsObject);
                enableBulkDeleteJSB.RestoreFromJSON(settingsJC["GeneralSettings"].AsObject);

                if (settingsJC["DesktopSettings"]["DefaultNewAtomPosRot"]!=null)desktopDefaultNewAtomPosRot.RestoreFromJSON(settingsJC["DesktopSettings"]["DefaultNewAtomPosRot"].AsObject);
                if (settingsJC["VRSettings"]["DefaultNewAtomPosRot"] != null) vrDefaultNewAtomPosRot.RestoreFromJSON(settingsJC["VRSettings"]["DefaultNewAtomPosRot"].AsObject);

                dockToVAMUIJSB.RestoreFromJSON(settingsJC["VRSettings"].AsObject);
                

                restoreInProgress = false;
            }
            else
            {
                foreach (int resourceType in EnumManifest.GetEnumsList(ResourceType.enumManifestName))
                {
                    ResourceSettings newResourceSettings = new ResourceSettings(resourceType);
                    resourceSettings.Add(resourceType, newResourceSettings);
                }
                SaveSettings();
            }

        }
    }

    public class ResourceSettings
    {
        public JSONStorableBool closeBrowserUIonResourceLoadJSB = new JSONStorableBool("closeBrowserUIonResourceLoad", false, SettingsBoolChange);
        public ResourceBrowserFilterSettings defaultFilterSettings;

        private int resourceType;
        public ResourceSettings(int resourceType)
        {
            defaultFilterSettings = new ResourceBrowserFilterSettings(resourceType);
            this.resourceType = resourceType;

        }

        public virtual JSONClass GetJSON()
        {
            JSONClass resourceSettingsJC = new JSONClass();
            closeBrowserUIonResourceLoadJSB.StoreJSON(resourceSettingsJC);
            resourceSettingsJC["defaultFilterSettings"] = defaultFilterSettings.GetJSON();

            resourceSettingsJC["combiTags"] = TagManifest.GetCombiTagsJSON(resourceType);
            resourceSettingsJC["hiddenTags"] = TagManifest.GetHiddenTagsJSON(resourceType);

            return resourceSettingsJC;
        }

        public virtual void RestoreJSON(JSONClass resourceSettingsJC)
        {
            closeBrowserUIonResourceLoadJSB.RestoreFromJSON(resourceSettingsJC);

            defaultFilterSettings.RestoreJSON(resourceSettingsJC["defaultFilterSettings"].AsObject);

            if (resourceSettingsJC["hiddenTags"] != null) TagManifest.RestoreHiddenTagsJSON(resourceType, resourceSettingsJC["hiddenTags"].AsArray);

            if (resourceSettingsJC["combiTags"] != null) TagManifest.RestoreCombiTagsJSON(resourceType, resourceSettingsJC["combiTags"].AsArray);
        }

        private static void SettingsBoolChange(bool val)
        {
            Settings.SaveSettings();
        }
    }

    public class ResourceBrowserFilterSettings
    {
        public JSONStorableMultiEnumStringChooser creatorJSMultiEnum;
        public JSONStorableString folderFilterJSS;
        public JSONStorableBool inludeSubFoldersJSB;
        public JSONStorableString resourceNameFilterJSS;

        public JSONStorableBool showHiddenJSB;
        public JSONStorableBool onlyFavouritesJSB;
        public JSONStorableBool onlyNewJSB;

        public JSONStorableMultiEnumStringChooser tagsFilterJSMultiEnum;
        public JSONStorableEnumStringChooser licenseFilterJSEnum;

        public JSONStorableEnumStringChooser selectModeJSEnum { get; set; }
        public JSONStorableEnumStringChooser sortModeJSEnum;

        public string resourceNameFilterQuick;

        public Folder selectedFolderFilter = null;

        public int resourceType { get; private set; }

        public bool combiTagFilterEnabled = false;
        public CombiTag combiTag;

        public ResourceBrowserFilterSettings(int _resourceType)
        {
            resourceType = _resourceType;
            

            var tagNames = TagManifest.GetTagNames(resourceType);
            tagsFilterJSMultiEnum = new JSONStorableMultiEnumStringChooser("tagsFilter", tagNames, tagNames, null, null, SettingsTagMainValChange, TagGroups.enumManifestName, SettingsTagTopValChange);

            showHiddenJSB = new JSONStorableBool("showHidden", false, SettingsBoolChange);
            onlyFavouritesJSB = new JSONStorableBool("onlyFavourites", false, SettingsBoolChange);
            onlyNewJSB = new JSONStorableBool("onlyNew", false, SettingsBoolChange);

            List<string> creators = CreatorManifest.GetAllCreatorNamesList();
            creatorJSMultiEnum = new JSONStorableMultiEnumStringChooser("creatorFilter", creators, creators, null, null, SettingsCreatorMainValChange, CreatorFilterChoices.enumManifestName, SettingsCreatorTopValChange);
            creatorJSMultiEnum.valTopEnumNoCallback = CreatorFilterChoices.allVARs;

            folderFilterJSS = new JSONStorableString("folderFilter", Const.GetResourceRootFolderNames(resourceType).First(),FolderFilterChange);
            selectedFolderFilter = FolderBrowser.GetFolder(folderFilterJSS.val, resourceType);

            inludeSubFoldersJSB = new JSONStorableBool("includeSubFolders",false, SettingsBoolChange);

            resourceNameFilterJSS = new JSONStorableString("resourceNameFilter", "");
            resourceNameFilterQuick = "";

            licenseFilterJSEnum = new JSONStorableEnumStringChooser("licenseFilter", LicenseType.enumManifestName, LicenseType.Any, "Package License", SettingsIntChange);

            selectModeJSEnum = new JSONStorableEnumStringChooser("selectMode", BrowserSelectMode.enumManifestName, BrowserSelectMode.single, "Select Mode", SettingsIntChange);
            sortModeJSEnum = new JSONStorableEnumStringChooser("sortMode", BrowserSortMode.enumManifestName, BrowserSortMode.newDateTime, "Order by", SettingsIntChange);
            
        }

        public ResourceBrowserFilterSettings(BAOrderedResourceFilter filters, CallbackBrowserFiltersUI parentFiltersUI=null)
        {
            resourceType = filters.resourceType;
            combiTag = new CombiTag("filterCombiTag", resourceType);

            var tagNames = TagManifest.GetTagNames(resourceType);
            if (parentFiltersUI != null) tagsFilterJSMultiEnum = new JSONStorableMultiEnumStringChooser("tagsFilter", tagNames, tagNames, null, null, parentFiltersUI.TagMainValCallback, TagGroups.enumManifestName, parentFiltersUI.TagTopValCallback);
            else tagsFilterJSMultiEnum = new JSONStorableMultiEnumStringChooser("tagsFilter", tagNames, tagNames, null, null, null, TagGroups.enumManifestName, null);
            tagsFilterJSMultiEnum.label = "Tags";
            if (filters.tagGroupType == BATagGroupTypeEnum.hiddenOnly) tagsFilterJSMultiEnum.valTopEnumNoCallback = TagGroups.hiddenOnly;
            else if (filters.tagGroupType == BATagGroupTypeEnum.noTags) tagsFilterJSMultiEnum.valTopEnumNoCallback = TagGroups.noTags;
            else if (filters.tagGroupType == BATagGroupTypeEnum.anyTags) tagsFilterJSMultiEnum.valTopEnumNoCallback = TagGroups.anyTags;
            else if (filters.tagGroupType == BATagGroupTypeEnum.noUserTags) tagsFilterJSMultiEnum.valTopEnumNoCallback = TagGroups.noUserTags;
            else if (filters.tagGroupType == BATagGroupTypeEnum.userDefault) tagsFilterJSMultiEnum.valNoCallback = Settings.resourceSettings[resourceType].defaultFilterSettings.tagsFilterJSMultiEnum.val;
            else if (filters.tagGroupType == BATagGroupTypeEnum.userTag) tagsFilterJSMultiEnum.mainValNoCallback = filters.userTagName;

            if (parentFiltersUI != null) showHiddenJSB = new JSONStorableBool("showHidden", false, parentFiltersUI.ShowHiddenFilterCallback);
            else showHiddenJSB = new JSONStorableBool("showHidden", false);
            if (filters.showHidden == BAShowHiddenEnum.includeHidden) showHiddenJSB.valNoCallback = true;
            else if (filters.showHidden == BAShowHiddenEnum.userDefault) showHiddenJSB.valNoCallback = Settings.resourceSettings[resourceType].defaultFilterSettings.showHiddenJSB.val;

            if (parentFiltersUI != null) onlyFavouritesJSB = new JSONStorableBool("onlyFavourites", false, parentFiltersUI.OnlyFavFilterCallback);
            else onlyFavouritesJSB = new JSONStorableBool("onlyFavourites", false);
            if (filters.onlyFav == BAOnlyFavoriteEnum.onlyFavorites) onlyFavouritesJSB.valNoCallback = true;
            else if (filters.onlyFav == BAOnlyFavoriteEnum.userDefault) onlyFavouritesJSB.valNoCallback = Settings.resourceSettings[resourceType].defaultFilterSettings.onlyFavouritesJSB.val;

            if (parentFiltersUI != null) onlyNewJSB = new JSONStorableBool("onlyNew", false, parentFiltersUI.OnlyNewFilterCallback);
            else onlyNewJSB = new JSONStorableBool("onlyNew", false);    
            if (filters.onlyNew == BAOnlyNewEnum.onlyNew) onlyNewJSB.valNoCallback = true;
            else if (filters.onlyNew == BAOnlyNewEnum.userDefault) onlyNewJSB.valNoCallback = Settings.resourceSettings[resourceType].defaultFilterSettings.onlyNewJSB.val;

            List<string> creators = CreatorManifest.GetAllCreatorNamesList();
            if (parentFiltersUI != null) creatorJSMultiEnum = new JSONStorableMultiEnumStringChooser("creatorFilter", creators, creators, null, null, parentFiltersUI.CreatorsMainValCallback, CreatorFilterChoices.enumManifestName, parentFiltersUI.CreatorsTopValCallback);
            else creatorJSMultiEnum = new JSONStorableMultiEnumStringChooser("creatorFilter", creators, creators, null, null, null, CreatorFilterChoices.enumManifestName, null);
            creatorJSMultiEnum.label = "Creator / Local";
            var localRootFolders = Const.GetResourceRootFolderNames(resourceType);
            folderFilterJSS = new JSONStorableString("folderFilter", localRootFolders.First(), FolderFilterChangeNoSave);
            if (parentFiltersUI != null) inludeSubFoldersJSB = new JSONStorableBool("includeSubFolders", false, parentFiltersUI.SubFoldersCallback);
            else inludeSubFoldersJSB = new JSONStorableBool("includeSubFolders", false);

            if (filters.creatorType == BACreatorTypeEnum.localFolders)
            {
                creatorJSMultiEnum.valTopEnumNoCallback = CreatorFilterChoices.localOnly;
                if (filters.localFolder != "") folderFilterJSS.valNoCallback = filters.localFolder;
                inludeSubFoldersJSB.valNoCallback = filters.includeSubFolders;
            }
            else if (filters.creatorType == BACreatorTypeEnum.allPackageCreators) creatorJSMultiEnum.valTopEnumNoCallback = CreatorFilterChoices.allVARs;
            else if (filters.creatorType == BACreatorTypeEnum.allLocalAndCreators) creatorJSMultiEnum.valTopEnumNoCallback = CreatorFilterChoices.all;
            else if (filters.creatorType == BACreatorTypeEnum.specificCreator) creatorJSMultiEnum.mainValNoCallback = filters.creatorName;
            else if (filters.creatorType == BACreatorTypeEnum.userDefault)
            {
                creatorJSMultiEnum.valNoCallback = Settings.resourceSettings[resourceType].defaultFilterSettings.creatorJSMultiEnum.val;
                if (creatorJSMultiEnum.valType == JSONStorableMultiEnumStringChooser.topEnumValFlag && creatorJSMultiEnum.valTopEnum == CreatorFilterChoices.localOnly)
                {
                    folderFilterJSS.valNoCallback = Settings.resourceSettings[resourceType].defaultFilterSettings.folderFilterJSS.val;
                    inludeSubFoldersJSB.valNoCallback = Settings.resourceSettings[resourceType].defaultFilterSettings.inludeSubFoldersJSB.val;
                }
            }
            
            selectedFolderFilter = FolderBrowser.GetFolder(folderFilterJSS.val, resourceType);            

            resourceNameFilterJSS = new JSONStorableString("resourceNameFilter", filters.resourceName);
            resourceNameFilterQuick = resourceNameFilterJSS.val.ToLower();

            if (parentFiltersUI != null) licenseFilterJSEnum = new JSONStorableEnumStringChooser("licenseFilter", LicenseType.enumManifestName, LicenseType.Any, "Package License", parentFiltersUI.LicenseCallback);
            else licenseFilterJSEnum = new JSONStorableEnumStringChooser("licenseFilter", LicenseType.enumManifestName, LicenseType.Any, "Package License", null);
            if (filters.varLicenseType == BALicenseTypeEnum.userDefault) licenseFilterJSEnum.valNoCallback = Settings.resourceSettings[resourceType].defaultFilterSettings.licenseFilterJSEnum.val;
            else licenseFilterJSEnum.valNoCallback = filters.varLicenseType;

            if (parentFiltersUI != null) selectModeJSEnum = new JSONStorableEnumStringChooser("selectMode", BrowserSelectMode.enumManifestName, BrowserSelectMode.instant, "Select Mode", parentFiltersUI.SelectModeCallback);
            else selectModeJSEnum = new JSONStorableEnumStringChooser("selectMode", BrowserSelectMode.enumManifestName, BrowserSelectMode.instant, "Select Mode", null);

            if (parentFiltersUI != null) sortModeJSEnum = new JSONStorableEnumStringChooser("sortMode", BrowserSortMode.enumManifestName, BrowserSortMode.newDateTime, "Order by", parentFiltersUI.SortByCallback );
            else sortModeJSEnum = new JSONStorableEnumStringChooser("sortMode", BrowserSortMode.enumManifestName, BrowserSortMode.newDateTime, "Order by");
            if (filters.resourceOrder == BAResultsSortModeEnum.userDefault) sortModeJSEnum.valNoCallback = Settings.resourceSettings[resourceType].defaultFilterSettings.sortModeJSEnum.val;
            else sortModeJSEnum.valNoCallback = filters.resourceOrder;

        }

        public ResourceBrowserFilterSettings(BrowserFiltersUI parentFiltersUI, int _resourceType)
        {
            resourceType = _resourceType;
            combiTag = new CombiTag("filterCombiTag", resourceType);

            var tagNames = TagManifest.GetTagNames(resourceType);
            tagsFilterJSMultiEnum = new JSONStorableMultiEnumStringChooser("tagsFilter", tagNames, tagNames, null, null, parentFiltersUI.TagMainValCallback, TagGroups.enumManifestName, parentFiltersUI.TagTopValCallback);
            tagsFilterJSMultiEnum.valNoCallback = Settings.resourceSettings[resourceType].defaultFilterSettings.tagsFilterJSMultiEnum.val;
            tagsFilterJSMultiEnum.label = "Tags";          

            showHiddenJSB = new JSONStorableBool("showHidden", Settings.resourceSettings[resourceType].defaultFilterSettings.showHiddenJSB.val, parentFiltersUI.ShowHiddenFilterCallback);
            onlyFavouritesJSB = new JSONStorableBool("onlyFavourites", Settings.resourceSettings[resourceType].defaultFilterSettings.onlyFavouritesJSB.val, parentFiltersUI.OnlyFavFilterCallback);
            onlyNewJSB = new JSONStorableBool("onlyNew", Settings.resourceSettings[resourceType].defaultFilterSettings.onlyNewJSB.val, parentFiltersUI.OnlyNewFilterCallback);

            List<string> creators = CreatorManifest.GetAllCreatorNamesList();
            creatorJSMultiEnum = new JSONStorableMultiEnumStringChooser("creatorFilter", creators, creators, null, null, parentFiltersUI.CreatorsMainValCallback, CreatorFilterChoices.enumManifestName, parentFiltersUI.CreatorsTopValCallback);
            creatorJSMultiEnum.valNoCallback = Settings.resourceSettings[resourceType].defaultFilterSettings.creatorJSMultiEnum.val;
            creatorJSMultiEnum.label = "Creator / Local";

            selectedFolderFilter = Settings.resourceSettings[resourceType].defaultFilterSettings.selectedFolderFilter;
            folderFilterJSS = new JSONStorableString("folderFilter", selectedFolderFilter.GetFolderPath());
            inludeSubFoldersJSB = new JSONStorableBool("includeSubFolders", Settings.resourceSettings[resourceType].defaultFilterSettings.inludeSubFoldersJSB.val, parentFiltersUI.SubFoldersCallback);

            resourceNameFilterJSS = new JSONStorableString("resourceNameFilter", "");
            resourceNameFilterQuick = resourceNameFilterJSS.val.ToLower();

            licenseFilterJSEnum = new JSONStorableEnumStringChooser("licenseFilter", LicenseType.enumManifestName, Settings.resourceSettings[resourceType].defaultFilterSettings.licenseFilterJSEnum.val, "Package License", parentFiltersUI.LicenseCallback);

            selectModeJSEnum = new JSONStorableEnumStringChooser("selectMode", BrowserSelectMode.enumManifestName, Settings.resourceSettings[resourceType].defaultFilterSettings.selectModeJSEnum.val, "Select Mode", parentFiltersUI.SelectModeCallback);
            sortModeJSEnum = new JSONStorableEnumStringChooser("sortMode", BrowserSortMode.enumManifestName, Settings.resourceSettings[resourceType].defaultFilterSettings.sortModeJSEnum.val, "Order by", parentFiltersUI.SortByCallback);
            
        }
        private void SettingsTagMainValChange(string i)
        {
            RefreshCreatorFilters();
            Settings.SaveSettings();
        }

        private void SettingsTagTopValChange(int i)
        {
            RefreshCreatorFilters();
            Settings.SaveSettings();
        }
        private void SettingsCreatorMainValChange(string i)
        {
            RefreshLicenseFilters();
            Settings.SaveSettings();
        }

        private void SettingsCreatorTopValChange(int i)
        {
            RefreshLicenseFilters();
            Settings.SaveSettings();
        }

        public virtual JSONClass GetJSON()
        {
            JSONClass defaultFilterSettingsJC = new JSONClass();
            creatorJSMultiEnum.StoreJSON(defaultFilterSettingsJC);

            showHiddenJSB.StoreJSON(defaultFilterSettingsJC);
            onlyFavouritesJSB.StoreJSON(defaultFilterSettingsJC);
            onlyNewJSB.StoreJSON(defaultFilterSettingsJC);

            tagsFilterJSMultiEnum.StoreJSON(defaultFilterSettingsJC);
            licenseFilterJSEnum.StoreJSON(defaultFilterSettingsJC);

            selectModeJSEnum.StoreJSON(defaultFilterSettingsJC);
            sortModeJSEnum.StoreJSON(defaultFilterSettingsJC);

            folderFilterJSS.valNoCallback = selectedFolderFilter.GetFolderPath();
            folderFilterJSS.StoreJSON(defaultFilterSettingsJC);
            inludeSubFoldersJSB.StoreJSON(defaultFilterSettingsJC);

            return defaultFilterSettingsJC;
        }

        public virtual void RestoreJSON(JSONClass defaultFilterSettingsJC)
        {
            creatorJSMultiEnum.RestoreFromJSON(defaultFilterSettingsJC);

            showHiddenJSB.RestoreFromJSON(defaultFilterSettingsJC);
            onlyFavouritesJSB.RestoreFromJSON(defaultFilterSettingsJC);
            onlyNewJSB.RestoreFromJSON(defaultFilterSettingsJC);

            tagsFilterJSMultiEnum.RestoreFromJSON(defaultFilterSettingsJC);
            licenseFilterJSEnum.RestoreFromJSON(defaultFilterSettingsJC);

            selectModeJSEnum.RestoreFromJSON(defaultFilterSettingsJC);
            sortModeJSEnum.RestoreFromJSON(defaultFilterSettingsJC);

            if (defaultFilterSettingsJC["folderFilter"]!=null) folderFilterJSS.RestoreFromJSON(defaultFilterSettingsJC);
            selectedFolderFilter = FolderBrowser.GetFolder(folderFilterJSS.val, resourceType);
            inludeSubFoldersJSB.RestoreFromJSON(defaultFilterSettingsJC);
        }

        private void FolderFilterChangeNoSave(string val)
        {
            selectedFolderFilter = FolderBrowser.GetFolder(val, resourceType);
        }

        private void FolderFilterChange(string val)
        {
            selectedFolderFilter = FolderBrowser.GetFolder(val, resourceType);
            Settings.SaveSettings();
        }

        private void SettingsBoolChange(bool val)
        {
            Settings.SaveSettings();
        }
        private void SettingsStringChange(string val)
        {
            Settings.SaveSettings();
        }

        private void SettingsIntChange(int val)
        {
            Settings.SaveSettings();
        }

        public void RefreshTagFilters()
        {
            List<string> tags = TagManifest.GetTagNames(resourceType);
            tagsFilterJSMultiEnum.SetMainChoices(tags);
            if (tagsFilterJSMultiEnum.valType == JSONStorableMultiEnumStringChooser.mainValFlag)
            {
                if (!tags.Contains(tagsFilterJSMultiEnum.mainVal)) tagsFilterJSMultiEnum.valTopEnum = TagGroups.anyTags;
            }

            List<int> ignoreTagGroups = new List<int>();
            var tagCategories = TagManifest.GetTagCategories(resourceType).ToList();
            tagCategories.Remove(TagCategory.userDefined);
            tagCategories.Remove(TagCategory.combi);
            if (tagCategories.Count == 0) ignoreTagGroups.Add(TagGroups.noUserTags);

            tagsFilterJSMultiEnum.SetTopEnum(TagGroups.enumManifestName, ignoreTagGroups);

            if (tagsFilterJSMultiEnum.valType == JSONStorableMultiEnumStringChooser.topEnumValFlag)
            {
                if (ignoreTagGroups.Contains(tagsFilterJSMultiEnum.valTopEnum)) tagsFilterJSMultiEnum.valTopEnum = TagGroups.anyTags;
            }
        }

        public void RefreshCreatorFilters()
        {
            HashSet<ResourceVersionGroupEntry> filteredRVGEs = new HashSet<ResourceVersionGroupEntry>(ResourceManifest.GetTagFilteredResourceGroups(this));


            CreatorManifest.UpdateCreatorChoices(creatorJSMultiEnum, filteredRVGEs);           
        }

        public void RefreshLicenseFilters()
        {
            if (creatorJSMultiEnum.valType!=JSONStorableMultiEnumStringChooser.topEnumValFlag || creatorJSMultiEnum.valTopEnum == CreatorFilterChoices.allVARs)
            {
                licenseFilterJSEnum.SetEnumChoices(LicenseType.enumManifestName, ResourceManifest.GetLicenseTypesByInverseFilters(this));
            }
        }
    }
}
