using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using UnityEngine.UI;

namespace JayJayWon
{
    public class CallbackBrowserFiltersUI:BrowserFiltersUI
    {
        private JSONStorableString browserTitleJSS = new JSONStorableString("", "Select a Resource");
        private ResourceBrowserFilterSettings callbackFilter ;

        private GameObject infoTextBoxGO;
        public override ResourceBrowserFilterSettings currentFilterSettings {
             get { return callbackFilter; } }
        public CallbackBrowserFiltersUI(BrowserUI _parentBrowser):base(_parentBrowser)
        {

        }
        public override void InitUI(GameObject _filtersBoxGO)
        {
            base.InitUI(_filtersBoxGO);
            resourceTypePopupUID.gameObject.SetActive(false);
            selectModePopup.gameObject.SetActive(false);
            if (infoTextBoxGO!=null) GameObject.Destroy(infoTextBoxGO);
            infoTextBoxGO = UI.CreateBasicTextBox(_filtersBoxGO, browserTitleJSS, 117f, 47f,24, 0, -2f, AnchorPresets.topLeft,backgoundColor: new Color(0.25f, 0.25f, 0.5f, 0.95f));
        }
        public void InitCallback(ResourceBrowserFilterSettings filter, string browserTitle, GameObject _filtersBoxGO)
        {
            filtersBoxGO = _filtersBoxGO;
            callbackFilter = filter;
            InitUI(filtersBoxGO);
            resourceType = callbackFilter.resourceType;
            if (browserTitle == "") browserTitleJSS.val = "Select a Resource";
            else browserTitleJSS.val = browserTitle;
        }

    }
    public class BrowserFiltersUI
    {
        private JSONStorableEnumStringChooser resourceTypeJSEnum;
        public int resourceType
        {
            get
            {
                return resourceTypeJSEnum.val;
            }
            set
            {
                resourceTypeJSEnum.val = value;
            }
        }
        public string resourceTypeName
        {
            get
            {
                return resourceTypeJSEnum.displayVal;
            }
        }


        public virtual ResourceBrowserFilterSettings currentFilterSettings {  get { return filterSettingsPerResourceType[resourceType]; } }
        public JSONStorableMultiEnumStringChooser creatorJSMultiEnum { get { return currentFilterSettings.creatorJSMultiEnum; } }

        public JSONStorableString folderFilterJSS { get { return currentFilterSettings.folderFilterJSS; } }

        public Folder selectedFolderFilter
        {
            get { return currentFilterSettings.selectedFolderFilter; }
            set { currentFilterSettings.selectedFolderFilter = value; }
        }

        public JSONStorableBool inludeSubFoldersJSB { get { return currentFilterSettings.inludeSubFoldersJSB; } }
        public JSONStorableString resourceNameFilterJSS { get { return currentFilterSettings.resourceNameFilterJSS; } }

        public JSONStorableBool showHiddenJSB { get { return currentFilterSettings.showHiddenJSB; } }
        public JSONStorableBool onlyFavouritesJSB { get { return currentFilterSettings.onlyFavouritesJSB; } }
        public JSONStorableBool onlyNewJSB { get { return currentFilterSettings.onlyNewJSB; } }

        public JSONStorableMultiEnumStringChooser tagsFilterJSMultiEnum { get { return currentFilterSettings.tagsFilterJSMultiEnum; } }
        public JSONStorableEnumStringChooser licenseFilterJSEnum { get { return currentFilterSettings.licenseFilterJSEnum; } }

        public JSONStorableEnumStringChooser selectModeJSEnum { get { if (currentFilterSettings == null) return null; return currentFilterSettings.selectModeJSEnum; } }
        public JSONStorableEnumStringChooser sortModeJSEnum { get { return currentFilterSettings.sortModeJSEnum; } }

        public string resourceNameFilterQuick
        {
            get
            { return currentFilterSettings.resourceNameFilterQuick; }
            set { currentFilterSettings.resourceNameFilterQuick = value; }
        }

        public bool isCombiTagFilterActive
        {
            get
            {
                if (currentFilterSettings==null) return false;
                return currentFilterSettings.combiTagFilterEnabled;
            }
            set
            {
                currentFilterSettings.combiTagFilterEnabled = value;
            }
        }

        public CombiTag combiTagFilter
        {
            get
            {
                return currentFilterSettings.combiTag;
            }
        }

        protected UIDynamicPopup resourceTypePopupUID;
        private UIDynamicButton refreshButtonUID;
        private UIDynamicPopup tagPopupUID;
        private UIDynamicButton combiTagEnableButtonUID;
        private UIDynamicButton combiTagSummaryButtonUID;
        private UIDynamicPopup creatorPopupUID;

        private UIDynamicPopup licensePopupUID;

        private UIDynamicToggle hiddenUID;
        private UIDynamicToggle onlyFavUID;
        private UIDynamicToggle onlyNewUID;
        private UIDynamicPopup sortModePopup;
        protected UIDynamicPopup selectModePopup;

        private UIDynamicButton clearResourceNameFilterButton;
        private UIDynamicButton resetCreatorFilterButton;
        private UIDynamicButton openFolderBrowserButtonUID;

        private UIDynamicTextField resourceNameTextUID;
        private UIDynamicTextField folderTextUID;
        private UIDynamicToggle incSubFoldersToggleUID;

        public Dictionary<int, ResourceBrowserFilterSettings> filterSettingsPerResourceType = new Dictionary<int, ResourceBrowserFilterSettings>();

        protected GameObject filtersBoxGO;
        private GameObject folderBrowserBoxGO;
        private GameObject combiTagFilterBoxGO;

        private FolderBrowserUI folderBrowserUI;
        private CombiTagFilterUI combiTagFilterUI;

        public BrowserUI parentBrowser;
        public BrowserFiltersUI(BrowserUI _parentBrowser)
        {
            parentBrowser = _parentBrowser;
            resourceTypeJSEnum = new JSONStorableEnumStringChooser("resourceType", ResourceType.enumManifestName, Settings.defaultResourceTypeJESC.val, "Resource Type", ResourceTypeFilterCallback);

            List<int> resourceTypesToExclude = new List<int>();
            if (!PC.patreonContentEnabled)
            {
                resourceTypesToExclude = EnumManifest.GetEnumsList(ResourceType.enumManifestName);
                resourceTypesToExclude.Remove(ResourceType.scene);
                resourceTypesToExclude.Remove(ResourceType.presetAppearance);
            }

            resourceTypeJSEnum.SetEnumChoices(ResourceType.enumManifestName, resourceTypesToExclude);

            foreach (int resourceType in EnumManifest.GetEnumsList(ResourceType.enumManifestName))
            {
                filterSettingsPerResourceType.Add(resourceType, new ResourceBrowserFilterSettings(this, resourceType));
            }
        }

        public virtual void InitUI(GameObject _filtersBoxGO)
        {
            filtersBoxGO = _filtersBoxGO;
            if (resourceTypePopupUID != null)
            {
                resourceTypePopupUID.popup.visible = false;
                GameObject.Destroy(resourceTypePopupUID.gameObject);
            }
            if (refreshButtonUID != null) GameObject.Destroy(refreshButtonUID.gameObject);
            if (openFolderBrowserButtonUID != null) GameObject.Destroy(openFolderBrowserButtonUID.gameObject);

            if (creatorPopupUID != null)
            {
                creatorPopupUID.popup.visible = false;
                GameObject.Destroy(creatorPopupUID.gameObject);
            }
            if (folderTextUID != null) GameObject.Destroy(folderTextUID.gameObject);
            if (incSubFoldersToggleUID != null) GameObject.Destroy(incSubFoldersToggleUID.gameObject);
            if (licensePopupUID != null)
            {
                licensePopupUID.popup.visible = false;
                GameObject.Destroy(licensePopupUID.gameObject);
            }
            if (hiddenUID != null) GameObject.Destroy(hiddenUID.gameObject);
            if (onlyFavUID != null) GameObject.Destroy(onlyFavUID.gameObject);
            if (onlyNewUID != null) GameObject.Destroy(onlyNewUID.gameObject);
            if (sortModePopup != null)
            {
                sortModePopup.popup.visible = false;
                GameObject.Destroy(sortModePopup.gameObject);
            }
            if (selectModePopup != null)
            {
                selectModePopup.popup.visible = false;
                GameObject.Destroy(selectModePopup.gameObject);
            }
            if (clearResourceNameFilterButton != null) GameObject.Destroy(clearResourceNameFilterButton.gameObject);
            if (resourceNameTextUID != null) GameObject.Destroy(resourceNameTextUID.gameObject);
            if (resetCreatorFilterButton != null) GameObject.Destroy(resetCreatorFilterButton.gameObject);
            if (folderBrowserBoxGO != null) GameObject.Destroy(folderBrowserBoxGO);
            if (combiTagFilterBoxGO != null) GameObject.Destroy(combiTagFilterBoxGO);

            resourceTypePopupUID = UI.CreatePopup(filtersBoxGO, resourceTypeJSEnum, 160f, 30f, null, 20, 0f, 0f, AnchorPresets.topLeft);
            parentBrowser.ApplyHelp(resourceTypePopupUID, "Select a type of VAM resource to apply as a filter to the Browser results.");
            RefreshCombiTagUI();

            creatorPopupUID = UI.CreateFilterablePopup(filtersBoxGO, creatorJSMultiEnum, 160f, 30f, null, 20, 330f, 0f, 300f, AnchorPresets.topLeft);
            creatorPopupUID.popup.onOpenPopupHandlers += RefreshCurrentCreatorFilters;
            parentBrowser.ApplyHelp(creatorPopupUID, "Select a Creator to filter Browser results by a specific VAR package creator. Non VAR packaged resources can be selected with the 'Local' option.");

            licensePopupUID = UI.CreatePopup(filtersBoxGO, licenseFilterJSEnum, 115f, 30f, null, 20, -35f, 0f, AnchorPresets.topRight);
            licensePopupUID.popup.onOpenPopupHandlers += RefreshCurrentLicenseFilters;
            parentBrowser.ApplyHelp(licensePopupUID, "Select a License type to filter Browser results by the license type applicable to the containing VAR package.");


            refreshButtonUID = UI.CreateUIButton(filtersBoxGO, 33f, 30f, "Rescan for new VARs", 16, 0f, 0f, AnchorPresets.topRight);
            refreshButtonUID.button.image.sprite = null;
            refreshButtonUID.buttonColor = Color.blue;
            refreshButtonUID.textColor = Color.white;

            refreshButtonUID.button.onClick.AddListener(delegate
            {
                SuperController.singleton.StartCoroutine(RefreshContents(refreshButtonUID));
            });
            parentBrowser.ApplyHelp(refreshButtonUID, "Scans for new VARs in the AddonPackages folder. Also updates Hidden & Favourite states of resources changed in VAM UI.");

            selectModePopup = UI.CreatePopup(filtersBoxGO, selectModeJSEnum, 120f, 20f, null, 20, 0f, -30f, AnchorPresets.topLeft, labelProportion: 0.55f);
            parentBrowser.ApplyHelp(selectModePopup, "Instant Mode: the default action (e.g. Load Scene) will be performed instantly when selecting a resource in the browser. Single Mode: only one resource is selectable at a time. Multi Mode: many resources can be selected and modified simultaneously.");

            hiddenUID = UI.CreateToggle(filtersBoxGO, showHiddenJSB, 85f, 20f, "Show Hidden", 20, 120f, -30f, AnchorPresets.topLeft);
            parentBrowser.ApplyHelp(hiddenUID, "Show resources that are marked as Hidden in the resource browser.");
            onlyFavUID = UI.CreateToggle(filtersBoxGO, onlyFavouritesJSB, 90f, 20f, "Only Favorites", 20, 205f, -30f, AnchorPresets.topLeft);
            parentBrowser.ApplyHelp(onlyFavUID, "Only show resources that are marked as Favourite in the resource browser.");

            onlyNewUID = UI.CreateToggle(filtersBoxGO, onlyNewJSB, 70f, 20f, "Only New", 20, 295f, -30f, AnchorPresets.topLeft);
            parentBrowser.ApplyHelp(onlyNewUID, "Only show resources that are marked as New or Updated in the resource browser.");

            resourceNameTextUID = UI.CreateTextBox(filtersBoxGO, resourceNameFilterJSS, 150f, 20f, 20, 365f, -30f, AnchorPresets.topLeft, true, InputField.LineType.MultiLineNewline, "Search Resource...", vScrollBarWidth: 0f, textEntryChangeCallback: ResourceNameFilterCallback);
            resourceNameTextUID.UItext.alignment = TextAnchor.MiddleLeft;
            SetResourceNameFilterHelpText();

            clearResourceNameFilterButton = UI.CreateUIButton(filtersBoxGO, 16f, 16f, "X", 20, 497f, -32f, AnchorPresets.topLeft);
            clearResourceNameFilterButton.button.onClick.AddListener(delegate { resourceNameFilterJSS.val = ""; });
            parentBrowser.ApplyHelp(clearResourceNameFilterButton, "Clear the Resource Name search field of user entered text.");
            if (resourceNameFilterJSS.val == "") clearResourceNameFilterButton.gameObject.SetActive(false);

            resetCreatorFilterButton = UI.CreateUIButton(filtersBoxGO, 16f, 16f, "X", 20, 472.5f, -13.3f, AnchorPresets.topLeft);
            resetCreatorFilterButton.button.onClick.AddListener(delegate { creatorJSMultiEnum.val = Settings.resourceSettings[resourceType].defaultFilterSettings.creatorJSMultiEnum.val; });
            parentBrowser.ApplyHelp(resetCreatorFilterButton, "Reset the Creator/Local filter to it's default value");
            if (creatorJSMultiEnum.valType != JSONStorableMultiEnumStringChooser.mainValFlag || creatorJSMultiEnum.val == Settings.resourceSettings[resourceType].defaultFilterSettings.creatorJSMultiEnum.val) resetCreatorFilterButton.gameObject.SetActive(false);

            sortModePopup = UI.CreatePopup(filtersBoxGO, sortModeJSEnum, 125f, 20f, null, 20, 0f, -30f, AnchorPresets.topRight, labelProportion: 0.35f, popupRowHeight: 24f);
            parentBrowser.ApplyHelp(sortModePopup, "Order Browser resource results Alphabetically, by Package and Folder or by Date. New dates are when items are first detected. For packaged items, the Updated date is when the highest VAR version was detected. Updated and New dates are the same for local resources.");

            combiTagFilterBoxGO = UI.AddChildGOImage(filtersBoxGO, new Color(0.2f, 0.2f, 0.2f, 0.95f), AnchorPresets.topRight, 450f, 275f, new Vector2(-160f, -30f));

            combiTagFilterUI = new CombiTagFilterUI(combiTagFilterBoxGO, parentBrowser);
            combiTagFilterBoxGO.SetActive(false);

            float folderBrowserHeight = Mathf.Min(500f, parentBrowser.screenHeight - parentBrowser.resourceBrowserTopOffset);
            folderBrowserBoxGO = UI.CreateVScrollableContent(filtersBoxGO, new Color(0.2f, 0.2f, 0.2f, 0.95f), AnchorPresets.topRight, 350f, folderBrowserHeight, new Vector2(-20f, -30f));
            folderBrowserBoxGO.SetActive(false);
            folderBrowserUI = new FolderBrowserUI(folderBrowserBoxGO, FolderBrowserType.filterSelect, FolderFilterSelected, parentBrowser);

            folderTextUID = UI.CreateTextBox(filtersBoxGO, folderFilterJSS, 100f, 15f, 20, -50f, 0f, AnchorPresets.topRight, false, InputField.LineType.SingleLine, vScrollBarWidth: 0f);
            folderTextUID.UItext.horizontalOverflow = HorizontalWrapMode.Overflow;
            folderTextUID.UItext.alignment = TextAnchor.MiddleLeft;
            parentBrowser.ApplyHelp(folderTextUID, "Current folder filter: " + selectedFolderFilter.GetFolderPath());

            incSubFoldersToggleUID = UI.CreateToggle(filtersBoxGO, inludeSubFoldersJSB, 100f, 15f, "Include subfolders", 20, -50f, -15f, AnchorPresets.topRight, 15f);
            parentBrowser.ApplyHelp(incSubFoldersToggleUID, "Include resources from sub-folders of the selected folder.");

            openFolderBrowserButtonUID = UI.CreateUIButton(filtersBoxGO, 15f, 30f, "V", 20, -35f, 0, AnchorPresets.topRight, false);
            parentBrowser.ApplyHelp(openFolderBrowserButtonUID, "Open the Folder Browser to select a folder for filtering resources");
            openFolderBrowserButtonUID.button.onClick.AddListener(delegate
            {
                if (!folderBrowserBoxGO.activeSelf) SetFolderBrowserActive();
                else SetFolderBrowserInActive();
            });
            RefreshLicenseFolderFilterVisibility();
            RefreshRescanButton();
        }

        public void ToggleCombiTagUI()
        {
            if (combiTagFilterBoxGO.activeSelf) combiTagFilterBoxGO.SetActive(false);
            else
            {
                combiTagFilterBoxGO.SetActive(true);
                combiTagFilterUI.RefreshUI(resourceType, combiTagFilter);
            }
        }

        public void RefreshCombiTagUI()
        {
            if (tagPopupUID != null)
            {
                tagPopupUID.popup.visible = false;
                GameObject.Destroy(tagPopupUID.gameObject);
            }
            if (combiTagEnableButtonUID != null) GameObject.Destroy(combiTagEnableButtonUID.gameObject);
            if (combiTagSummaryButtonUID != null) GameObject.Destroy(combiTagSummaryButtonUID.gameObject);

            if (isCombiTagFilterActive && PC.patreonContentEnabled)
            {
                combiTagSummaryButtonUID = UI.CreateUIButton(filtersBoxGO, 145f, 27f, combiTagFilter.description, 20, 165f, -2f, AnchorPresets.topLeft, true);
                combiTagSummaryButtonUID.button.onClick.AddListener(delegate {
                    ToggleCombiTagUI();

                });
                parentBrowser.ApplyHelp(combiTagSummaryButtonUID, "Toggle disaplying the Tag Combination filter setup window.");


                combiTagEnableButtonUID = UI.CreateUIButton(filtersBoxGO, 15f, 30f, "X", 20, 165f + 145f, 0f, AnchorPresets.topLeft);
                parentBrowser.ApplyHelp(combiTagEnableButtonUID, "Cancel tag combination filtering and use single tag filtering");
                combiTagEnableButtonUID.button.onClick.AddListener(delegate
                {
                    isCombiTagFilterActive = false;
                    combiTagFilterBoxGO.SetActive(false);
                    RefreshCombiTagUI();
                    parentBrowser.RefreshFilterResults();
                });

            }
            else
            {

                tagPopupUID = UI.CreateFilterablePopup(filtersBoxGO, tagsFilterJSMultiEnum, 145f, 30f, null, 20, 165f, 0f, 150f, AnchorPresets.topLeft);
                tagPopupUID.popup.onOpenPopupHandlers += RefreshCurrentTagFilters;
                parentBrowser.ApplyHelp(tagPopupUID, "Select a Tag to apply as a filter to the Browser results.");

                if (PC.patreonContentEnabled)
                {
                    combiTagEnableButtonUID = UI.CreateUIButton(filtersBoxGO, 15f, 30f, "V", 20, 165f + 145f, 0f, AnchorPresets.topLeft);
                    parentBrowser.ApplyHelp(combiTagEnableButtonUID, "Filter resources using a combination of Tags");
                    combiTagEnableButtonUID.button.onClick.AddListener(delegate
                    {
                        isCombiTagFilterActive = true;
                        if (folderBrowserBoxGO.activeSelf) SetFolderBrowserInActive();

                        combiTagFilterBoxGO.SetActive(true);
                        combiTagFilterUI.RefreshUI(resourceType, combiTagFilter);
                        RefreshCombiTagUI();
                        parentBrowser.RefreshFilterResults();
                    });
                }

            }
        }

        public void SetFolderBrowserActive()
        {
            selectedFolderFilter.ExpandParents();
            folderBrowserUI.SetNormalFolderMode();
            folderBrowserUI.RefreshUI(resourceType);
            folderBrowserBoxGO.SetActive(true);
        }
        public void SetFolderBrowserInActive()
        {
            folderBrowserBoxGO.SetActive(false);
        }

        public void FolderFilterSelected(Folder selectedFolder)
        {
            folderFilterJSS.val = selectedFolder.folderName;
            selectedFolderFilter = selectedFolder;
            if (!Settings.keepFolderBrowserOpenJSB.val) folderBrowserBoxGO.SetActive(false);
            if (selectedFolder.isRootFolder && folderFilterJSS.val.Length > 16) folderTextUID.UItext.alignment = TextAnchor.MiddleRight;
            else folderTextUID.UItext.alignment = TextAnchor.MiddleLeft;

            parentBrowser.ApplyHelp(folderTextUID, "Filter to local folder: " + selectedFolderFilter.GetFolderPath());

            currentFilterSettings.RefreshCreatorFilters();
            parentBrowser.RefreshFilterResults(true);
        }

        private void RefreshLicenseFolderFilterVisibility()
        {
            licensePopupUID.gameObject.SetActive(false);
            folderTextUID.gameObject.SetActive(false);
            incSubFoldersToggleUID.gameObject.SetActive(false);
            openFolderBrowserButtonUID.gameObject.SetActive(false);
            licensePopupUID.popup.visible = false;


            if (creatorJSMultiEnum.valType == JSONStorableMultiEnumStringChooser.mainValFlag || creatorJSMultiEnum.valTopEnum == CreatorFilterChoices.allVARs)
            {
                licensePopupUID.gameObject.SetActive(true);
                folderBrowserBoxGO.SetActive(false);
            }
            else if (creatorJSMultiEnum.valTopEnum == CreatorFilterChoices.localOnly)
            {
                folderTextUID.gameObject.SetActive(true);
                incSubFoldersToggleUID.gameObject.SetActive(true);
                openFolderBrowserButtonUID.gameObject.SetActive(true);
            }
            else folderBrowserBoxGO.SetActive(false);
        }

        public void SetResourceNameFilterHelpText()
        {
            if (Settings.resourceNameSearchModeJSESC.val == ResourceNameSearchMode.StartsWith) parentBrowser.ApplyHelp(resourceNameTextUID, "Filter resources where the resource name starts with the specified search text");
            else if (Settings.resourceNameSearchModeJSESC.val == ResourceNameSearchMode.Contains) parentBrowser.ApplyHelp(resourceNameTextUID, "Filter resources where the resource name contains the specified search text");
            else parentBrowser.ApplyHelp(resourceNameTextUID, "Filter resources where the resource name or its full file path contains the specified search text");
        }

        private IEnumerator RefreshContents(UIDynamicButton refreshButtonUID)
        {
            string tempLabel = refreshButtonUID.label;
            Color buttonCol = refreshButtonUID.buttonColor;
            refreshButtonUID.label = "Scan\n-ing";
            refreshButtonUID.buttonColor = Color.cyan;
            refreshButtonUID.textColor = Color.white;

            yield return new WaitForSeconds(0.01f);

            try
            {
                if ((creatorJSMultiEnum.valType == JSONStorableMultiEnumStringChooser.topEnumValFlag && creatorJSMultiEnum.valTopEnum != CreatorFilterChoices.localOnly) || creatorJSMultiEnum.valType != JSONStorableMultiEnumStringChooser.topEnumValFlag)
                {
                    VARPackageManifest.RescanPackages();

                }

                DateTime scanTime = DateTime.Now;

                if (creatorJSMultiEnum.valType == JSONStorableMultiEnumStringChooser.topEnumValFlag && (creatorJSMultiEnum.valTopEnum == CreatorFilterChoices.localOnly || creatorJSMultiEnum.valTopEnum == CreatorFilterChoices.all)) ResourceManifest.LocalResourcesInit(scanTime, new HashSet<int>());

                parentBrowser.RefreshFilterResults(true);

                refreshButtonUID.buttonText.text = tempLabel;
                refreshButtonUID.buttonColor = Color.blue;
            }
            catch (Exception e)
            {
                SuperController.LogError("BA.BrowserFiltersUI.RefreshContents: Exception caught:\n" + e);
            }
        }

        private void RefreshCurrentTagFilters()
        {
            currentFilterSettings.RefreshTagFilters();
        }
        private void RefreshCurrentCreatorFilters()
        {
            currentFilterSettings.RefreshCreatorFilters();
        }

        private void RefreshCurrentLicenseFilters()
        {
            currentFilterSettings.RefreshLicenseFilters();
        }

        private void RefreshRescanButton()
        {
            if (creatorJSMultiEnum.valType == JSONStorableMultiEnumStringChooser.mainValFlag || creatorJSMultiEnum.valTopEnum == CreatorFilterChoices.allVARs)
            {
                refreshButtonUID.label = "Rescan for new VARs";
                parentBrowser.ApplyHelp(refreshButtonUID, "Scans for new VARs in the AddonPackages folder. Also updates Hidden & Favourite states of resources changed in VAM UI.");
            }
            else
            {
                if (creatorJSMultiEnum.valTopEnum == CreatorFilterChoices.all)
                {
                    refreshButtonUID.label = "Rescan Local & VARs";
                    parentBrowser.ApplyHelp(refreshButtonUID, "Scans for new VARs in the AddonPackages folder and new (or removed) files in the local Saves and Custom folders. Also updates Hidden & Favourite states of resources changed in VAM UI.");
                }
                else
                {
                    refreshButtonUID.label = "Rescan Local files";
                    parentBrowser.ApplyHelp(refreshButtonUID, "Scans for new (or removed) files in the local Saves and Custom folders. Also updates Hidden & Favourite states of resources changed in VAM UI.");
                }
            }
        }

        public void CreatorsTopValCallback(int i)
        {
            RefreshLicenseFolderFilterVisibility();
            RefreshRescanButton();


            if (creatorJSMultiEnum.valType != JSONStorableMultiEnumStringChooser.mainValFlag || creatorJSMultiEnum.val == Settings.resourceSettings[resourceType].defaultFilterSettings.creatorJSMultiEnum.val) resetCreatorFilterButton.gameObject.SetActive(false);
            else resetCreatorFilterButton.gameObject.SetActive(true);
            currentFilterSettings.RefreshLicenseFilters();
            parentBrowser.RefreshFilterResults(true);
        }
        public void CreatorsMainValCallback(string i)
        {
            RefreshRescanButton();
            currentFilterSettings.RefreshLicenseFilters();

            if (creatorJSMultiEnum.valType != JSONStorableMultiEnumStringChooser.mainValFlag || creatorJSMultiEnum.val == Settings.resourceSettings[resourceType].defaultFilterSettings.creatorJSMultiEnum.val) resetCreatorFilterButton.gameObject.SetActive(false);
            else resetCreatorFilterButton.gameObject.SetActive(true);

            parentBrowser.RefreshFilterResults(true);
            RefreshLicenseFolderFilterVisibility();
        }

        public void TagTopValCallback(int i)
        {
            if (i == TagGroups.hiddenOnly)
            {
                showHiddenJSB.valNoCallback = true;
                ShowHiddenFilterCallback(true);
                return;
            }
            currentFilterSettings.RefreshCreatorFilters();
            parentBrowser.RefreshFilterResults(true);

        }
        public void TagMainValCallback(string i)
        {
            currentFilterSettings.RefreshCreatorFilters();
            parentBrowser.RefreshFilterResults(true);

        }

        public void ResourceNameFilterCallback(string i)
        {
            resourceNameFilterQuick = i.ToLower();
            if (resourceNameFilterQuick == "") clearResourceNameFilterButton.gameObject.SetActive(false);
            else clearResourceNameFilterButton.gameObject.SetActive(true);
            parentBrowser.RefreshFilterResults(true);

        }
        public void ResourceTypeFilterCallback(int value)
        {
            InitUI(filtersBoxGO);
            parentBrowser.RefreshFilterResults(true);
            parentBrowser.RefreshBrowserUILayout();

            parentBrowser.resourceActionsUI.RefreshUI();

            parentBrowser.resourceActionsUI.primaryActionUITab.RefreshTargetAtomPopup(true);
            parentBrowser.resourceSelectionsUI.RePopulateSelectionsBox();

        }
        public void ShowHiddenFilterCallback(bool value)
        {
            //            if (!value) VARPackageManifest.RefreshVARUserFilePrefs();
            currentFilterSettings.RefreshCreatorFilters();
            parentBrowser.RefreshFilterResults(true);
            parentBrowser.resourceActionsUI.RefreshUI();
        }
        public void OnlyFavFilterCallback(bool value)
        {
            //            if (value) VARPackageManifest.RefreshVARUserFilePrefs();
            currentFilterSettings.RefreshCreatorFilters();
            parentBrowser.RefreshFilterResults(true);
            parentBrowser.resourceActionsUI.RefreshUI();
        }

        public void SubFoldersCallback(bool value)
        {
            parentBrowser.RefreshFilterResults(true);
            parentBrowser.resourceActionsUI.RefreshUI();
        }
        public void OnlyNewFilterCallback(bool value)
        {
            currentFilterSettings.RefreshCreatorFilters();
            parentBrowser.RefreshFilterResults(true);
            parentBrowser.resourceActionsUI.RefreshUI();
        }

        public void SortByCallback(string i)
        {
            parentBrowser.RefreshFilterResults();
        }
        public void SelectModeCallback(int i)
        {
            parentBrowser.resourceSelectionsUI.SwitchSelectMode(resourceType, i);
            if (parentBrowser.resourceActionsUI.currentActionUITabType != ActionUITabType.infoTab) parentBrowser.resourceActionsUI.RefreshUI();
            parentBrowser.RefreshBrowserUILayout();
        }

        public void LicenseCallback(int i)
        {
            parentBrowser.RefreshFilterResults();
        }
    }


}
